@@ -12,7 +12,8 @@ use File::Basename;
 use Test::Builder;
 use File::Find;
 
-our $VERSION = '1.26';
+our $VERSION = '1.33';
+our $VERBOSE = 1;
 
 my $test      = Test::Builder->new();
 my $test_bool = 1;
@@ -36,28 +37,40 @@ sub import {
 }
 
 sub ok_manifest{
-    my ($hashref,$msg)    = @_;
+    my ($hashref,$msg) = @_;
     
     $test->plan(tests => 1) unless $plan;
     
     my $is_hashref = 1;
     $is_hashref = 0 unless ref($hashref);
     
-    $msg = $hashref unless $is_hashref;
+    unless ( $is_hashref ) {
+        $msg = $hashref;
+        $hashref = {};
+    }
+
+    my $tmp_path = dirname( File::Spec->rel2abs( $0 ) );
+
+    if ( $hashref->{file} ) {
+        $tmp_path = dirname $hashref->{file};
+    }
+    elsif ( $hashref->{dir} ) {
+        $tmp_path = $hashref->{dir};
+    }
     
     my $bool = 1;
-    my $home = Cwd::realpath( dirname ( File::Spec->rel2abs($0) ) );
+    my $home = Cwd::realpath( $tmp_path );
     my $manifest;
 
     my $counter = 0;
     while ( 1 ) {
-        my $tmp_home = Cwd::realpath( $home . '/..' );
-
-        last if $tmp_home eq $home || $counter++ == 20;
-        $home = $tmp_home;
-
         my $manifest_path = File::Spec->catfile( $home . '/MANIFEST' );
-        last if -e $manifest_path;
+        last if -f $manifest_path;
+
+        my $tmp_home = Cwd::realpath( File::Spec->catdir( $home, '..' ) );
+
+        last if !$tmp_home || $tmp_home eq $home || $counter++ == 20;
+        $home = $tmp_home;
     }
 
     eval { $manifest = Cwd::realpath( $home . '/MANIFEST' ); 1; };
@@ -66,8 +79,10 @@ sub ok_manifest{
     }
     
     my $skip;
-    eval { $skip     = Cwd::realpath( $home . '/MANIFEST.SKIP' ); 1; };
-    
+    my $skip_path = File::Spec->catfile( $home, 'MANIFEST.SKIP' );
+    eval { $skip = Cwd::realpath( $skip_path ) if -f $skip_path; 1; };
+
+    my @dup_files     = ();
     my @missing_files = ();
     my @files_plus    = ();
     my $arref         = ['/blib' , '/_build'];
@@ -101,7 +116,7 @@ sub ok_manifest{
         
         my $files_in_skip = _read_skip( $skip, \$msg, \$bool );
         last unless $files_in_skip;
-            
+
         my @files = _read_file( $fh );
         close $fh;
     
@@ -115,14 +130,16 @@ sub ok_manifest{
         for my $tfile(@files){
             $tfile = (split(/\s{2,}/,$tfile,2))[0];
             next unless -e $home . '/' . $tfile;
-            $tfile = Cwd::realpath($home . '/' . $tfile);
+            $tfile = File::Spec->rel2abs($home . '/' . $tfile);
         }
     
         my (@dir_files,%files_hash,%excluded);
         @files_hash{@files} = ();
     
-        find({no_chdir => 1,
-            wanted   => sub{
+        find({
+            no_chdir => 1,
+            follow   => 0,
+            wanted   => sub {
                 my $file         = $File::Find::name;
                 my $is_excluded  = _is_excluded(
                     $file,
@@ -133,7 +150,7 @@ sub ok_manifest{
                     $home,
                 );
                 
-                push(@dir_files,Cwd::realpath($file)) if -f $file and !$is_excluded;
+                push(@dir_files,File::Spec->rel2abs($file)) if -f $file and !$is_excluded;
                 
                 $excluded{$file} = 1 if -f $file and $is_excluded
             }
@@ -156,6 +173,10 @@ sub ok_manifest{
         delete $files_hash{$_} for keys %excluded;
         @files_plus = sort keys %files_hash;
         $bool = 0 if scalar @files_plus > 0;
+
+        my %seen_files = ();
+        @dup_files = map { 1==$seen_files{$_}++ ? $_ : () } @files;
+        $bool = 0 if scalar @dup_files > 0;
         
         } # close extra block
     }
@@ -164,20 +185,28 @@ sub ok_manifest{
                join(', ',@missing_files);
     my $plus = 'The following files are not part of distro but named in the MANIFEST file: '.
                join(', ',@files_plus);
+    my $dup  = 'The following files appeared more than once in the MANIFEST file: '.
+               join(', ',@dup_files);
     
-    $test->is_num($test_bool,$bool,$msg);
-    $test->diag($diag) if scalar @missing_files >= 1 and $test_bool == 1;
-    $test->diag($plus) if scalar @files_plus    >= 1 and $test_bool == 1;
+    $test->is_num($bool,$test_bool,$msg);
+    $test->diag($diag) if scalar @missing_files >= 1 and $test_bool == 1 and $VERBOSE;
+    $test->diag($plus) if scalar @files_plus    >= 1 and $test_bool == 1 and $VERBOSE;
+    $test->diag($dup)  if scalar @dup_files     >= 1 and $test_bool == 1 and $VERBOSE;
 }
 
 sub _read_file {
     my ($fh) = @_;
     
     my @files;
+    my $selftest = 0;
+
     while( my $fh_line = <$fh> ){
         chomp $fh_line;
         
+        $selftest++ if $fh_line =~ m{# MANIFEST for Test-CheckManifest};
+
         next if $fh_line =~ m{ \A \s* \# }x;
+        next if $selftest && $fh_line =~ m{# selftest};
         
         my ($file);
         
@@ -189,7 +218,7 @@ sub _read_file {
         }
 
         next unless $file;
-        
+
         push @files, $file;
     }
     
@@ -204,8 +233,10 @@ sub _not_ok_manifest{
 
 sub _is_excluded{
     my ($file,$dirref,$filter,$bool,$files_in_skip,$home) = @_;
-    my @excluded_files = qw(pm_to_blib Makefile META.yml Build pod2htmd.tmp
-                            pod2htmi.tmp Build.bat .cvsignore MYMETA.json);
+    my @excluded_files = qw(
+        pm_to_blib Makefile META.yml Build pod2htmd.tmp 
+        pod2htmi.tmp Build.bat .cvsignore MYMETA.json MYMETA.yml
+    );
 
     if ( $files_in_skip and 'ARRAY' eq ref $files_in_skip ) {
         (my $local_file = $file) =~ s{\Q$home\E/?}{};
@@ -251,26 +282,25 @@ sub _read_skip {
 
 1;
 
+__END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Test::CheckManifest - Check if your Manifest matches your distro
 
 =head1 VERSION
 
-version 1.26
+version 1.33
 
 =head1 SYNOPSIS
 
   use Test::CheckManifest;
   ok_manifest();
 
-=head1 DESCRIPTION
-
-C<Test::CheckManifest>
-
 =head2 EXPORT
 
 There is only one method exported: C<ok_manifest>
@@ -338,6 +368,10 @@ These files not:
 
 =back
 
+By default, C<ok_manifest> will look for the file C<MANIFEST> in the current working directory (which is how tests are traditionally run). If you wish to specify a different directory, you may pass the C<file> or C<dir> parameters, for example:
+
+  ok_manifest({dir => '/path/to/my/dist/'});
+
 =head1 EXCLUDING FILES
 
 Beside C<filter> and C<exclude> there is another way to exclude files:
@@ -353,18 +387,14 @@ he reported some bugs to RT.
 
 =head1 AUTHOR
 
-Renee Baecker <module@renee-baecker.de>
+Renee Baecker <reneeb@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2012 by Renee Baecker.
+This software is Copyright (c) 2018 by Renee Baecker.
 
 This is free software, licensed under:
 
   The Artistic License 2.0 (GPL Compatible)
 
 =cut
-
-
-__END__
-
